"""The 'moreworktoy' module provides new stuff for worktoy created for use
by the current project, but being too general to be included in the
project."""
#  AGPL-3.0 license
#  Copyright (c) 2024 Asger Jon Vistisen
from __future__ import annotations

from ._maybe import maybe
from ._maybe_type import maybeType
